# Requirements
## Hardware Requirements
- GPU: NVIDIA Quadro RTX 8000 (48GB Memory)
- CPU: Intel(R) Xeon(R) Gold 5120 CPU @ 2.20GHz
- Memory: 376GB
- SSD: 1T

## Software Dependency
- OS: Ubuntu 16.04 LTS
- Anaconda 3
- CUDA 10.2, CUDNN 7.6
- Python libraries: Python3.8, Pytorch 1.7.0

# Installation

This document contains detailed instructions for installing the necessary dependencies for **MixFormer**. 
The instrustions have been tested on an Ubuntu 16.04 system. 

## Quick dependency packages installation
- To prepare for the environment, run the script `install_pytorch17.sh`
``` shell
$ chmod a+x install_pytorch17.sh
$ conda create -n MixFormer python=3.8
$ conda activate MixFormer
$ ./install_pytorch17.sh
```
## Set project paths

Run the following command to set paths for this project

``` shell
$ python tracking/create_default_local_file.py --workspace_dir . --data_dir ./data --save_dir .
```

After running this command, you can also modify paths by editing these two files

``` shell
lib/train/admin/local.py  # paths about training
lib/test/evaluation/local.py  # paths about testing
```

## Run MixFormer

Download the networks from [google drive](https://drive.google.com/file/d/1FZSM_sahxfhI3J9Fc7PwKnVxoX0G9LAa/view?usp=sharing) and put them under the `$SAVE_DIR/models/` dir.

- In the terminal run a command to create workspace:
  `vot initialize <stack-name> --workspace <workspace-path>`

- Go to `workspace/` and modify some paths

- Fill in **paths** && **env_PATH** in `trackers.ini` to your real paths. 

  Example of  `trackers.ini`:
``` ini
[MixFormer-ViT-B]  # <tracker-name>
label = MixFormer-ViT-B
protocol = traxpython
command = submit_vit_base

paths = <set to root dir of the submitted source code, example: /home/user/project/MixFormer_submit>

env_PATH = /usr/bin:<set to lib dir of your cuda, example: /home/user/cuda-10.0/lib64/>:<set to bin dir of your conda env, example: /home/user/anaconda3/envs/MixFormer/bin>
```

- Run MixFormer and analyze results

**Note**: After download VOT2022-STB dataset, please move it to SSD or `/dev/shm` for improving I/O speed :)  
You can create a soft link called `sequences/` and put in under `workspace/` dir.

``` shell
$ conda activate MixFormer
$ vot evaluate --workspace <source code root>/workspace MixFormer-ViT-B
$ vot analysis --nocache <source code root>/workspace MixFormer-ViT-B
```